@@ -17,7 +17,7 @@ from function_scheduling_distributed_framework.publishers.sqla_queue_publisher i
 
 
 def get_publisher(queue_name, *, log_level_int=10, logger_prefix='', is_add_file_handler=True,
-                  clear_queue_within_init=False, is_add_publish_time=True,consuming_function: Callable = None,
+                  clear_queue_within_init=False, is_add_publish_time=True, consuming_function: Callable = None,
                   broker_kind=0):
     """
     :param queue_name:
@@ -26,7 +26,6 @@ def get_publisher(queue_name, *, log_level_int=10, logger_prefix='', is_add_file
     :param is_add_file_handler:
     :param clear_queue_within_init:
     :param is_add_publish_time:是否添加发布时间，以后废弃，都添加。
-    :param is_using_rpc_mode:是否使用rpc模式，发布端将可以获取消费端的结果。需要安装redis和额外的性能。
     :param consuming_function:消费函数，为了做发布时候的函数入参校验用的，如果不传则不做发布任务的校验，
                例如add 函数接收x，y入参，你推送{"x":1,"z":3}就是不正确的，函数不接受z参数。
     :param broker_kind: 中间件或使用包的种类。
@@ -34,27 +33,19 @@ def get_publisher(queue_name, *, log_level_int=10, logger_prefix='', is_add_file
     """
     all_kwargs = copy.deepcopy(locals())
     all_kwargs.pop('broker_kind')
-    if broker_kind == 0:
-        return RabbitmqPublisher(**all_kwargs)
-    elif broker_kind == 1:
-        return RabbitmqPublisherUsingRabbitpy(**all_kwargs)
-    elif broker_kind == 2:
-        return RedisPublisher(**all_kwargs)
-    elif broker_kind == 3:
-        return LocalPythonQueuePublisher(**all_kwargs)
-    elif broker_kind == 4:
-        return RabbitmqPublisherUsingAmqpStorm(**all_kwargs)
-    elif broker_kind == 5:
-        return MongoMqPublisher(**all_kwargs)
-    elif broker_kind == 6:
-        return PersistQueuePublisher(**all_kwargs)
-    elif broker_kind == 7:
-        return NsqPublisher(**all_kwargs)
-    elif broker_kind == 8:
-        return KafkaPublisher(**all_kwargs)
-    elif broker_kind == 9:
-        return RedisPublisher(**all_kwargs)
-    elif broker_kind == 10:
-        return SqlachemyQueuePublisher(**all_kwargs)
-    else:
+    broker_kind__publisher_type_map = {
+        0: RabbitmqPublisher,
+        1: RabbitmqPublisherUsingRabbitpy,
+        2: RedisPublisher,
+        3: LocalPythonQueuePublisher,
+        4: RabbitmqPublisherUsingAmqpStorm,
+        5: MongoMqPublisher,
+        6: PersistQueuePublisher,
+        7: NsqPublisher,
+        8: KafkaPublisher,
+        9: RedisPublisher,
+        10: SqlachemyQueuePublisher,
+    }
+    if broker_kind not in broker_kind__publisher_type_map:
         raise ValueError('设置的中间件种类数字不正确')
+    return broker_kind__publisher_type_map[broker_kind](**all_kwargs)
